import pandas as pd
import pytz
import pyilz.parse_land as parse_land


def get_timers_from_state(gamestate):
    """
    Returns the active, passive, and completed activities from the gamestate.

    Args:
        gamestate (str): The gamestate string.

    Returns:
        Tuple: A tuple containing three lists - active, passive, and completed activities.
    """
    _, _, ca, aa, completed = parse_land.parse_land(gamestate)
    return get_timers(ca, aa, completed)


def get_timers(ca, aa, completed, init=False):
    """
    Returns a dictionary of timers for the specified activities.

    Args:
        ca (pandas.DataFrame): A DataFrame containing information about current activities.
        aa (pandas.DataFrame): A DataFrame containing information about auto activities.
        completed (pandas.DataFrame): A DataFrame containing information about completed activities.
        init (bool): A flag indicating whether all timers should be marked as notified.

    Returns:
        dict: A dictionary of timers for the specified activities.
    """
    timers = {}
    if ca is not None:
        for i, row in ca.iterrows():
            end_time = row['currentActivity.EndTime'].replace(
                tzinfo=pytz.utc)
            end_unix = end_time.timestamp()
            cur_time = pd.Timestamp.now(tz='utc')
            diff_minutes = (end_time - cur_time).total_seconds() / 60

            # Calculate percentage based on currentActivity.StartTime
            # and currentActivity.EndTime
            start_time = row['currentActivity.StartTime'].replace(
                tzinfo=pytz.utc)
            start_unix = start_time.timestamp()
            total_time = (end_time - start_time).total_seconds() / 60
            elapsed_time = (cur_time - start_time).total_seconds() / 60
            percentage = elapsed_time / total_time * 100

            notified = False
            if init and diff_minutes <= 0:
                notified = True
            timers[f'{row["uid"]}-current'] = {'uid': row['uid'],
                                               'name': row['buildingTypeString'],
                                               'type': row['currentActivity.Type'],
                                               'minutes': diff_minutes,
                                               'notified': notified,
                                               'percentage': percentage,
                                               'end': end_unix,
                                               'start': start_unix}

    if aa is not None:
        for i, row in aa.iterrows():
            end_time = row['autoActivity.EndTime'].replace(tzinfo=pytz.utc)
            end_unix = end_time.timestamp()
            cur_time = pd.Timestamp.now(tz='utc')
            diff_minutes = (end_time - cur_time).total_seconds() / 60

            # Calculate percentage based on currentActivity.StartTime
            # and currentActivity.EndTime
            start_time = row['autoActivity.StartTime'].replace(
                tzinfo=pytz.utc)
            start_unix = start_time.timestamp()
            total_time = (end_time - start_time).total_seconds() / 60
            elapsed_time = (cur_time - start_time).total_seconds() / 60
            percentage = elapsed_time / total_time * 100

            notified = False
            if init and diff_minutes <= 0:
                notified = True
            timers[f'{row["uid"]}-auto'] = {'uid': row['uid'],
                                            'name': row['buildingTypeString'],
                                            'type': row['autoActivity.Type'],
                                            'minutes': diff_minutes,
                                            'notified': notified,
                                            'percentage': percentage,
                                            'end': end_unix,
                                            'start': start_unix}

    if completed is not None:
        for i, row in completed.iterrows():
            end_time = row['completedActivity.EndTime'].replace(
                tzinfo=pytz.utc)
            end_unix = end_time.timestamp()
            cur_time = pd.Timestamp.now(tz='utc')
            diff_minutes = (end_time - cur_time).total_seconds() / 60

            # Calculate percentage based on currentActivity.StartTime
            # and currentActivity.EndTime
            start_time = row['completedActivity.StartTime'].replace(
                tzinfo=pytz.utc)
            start_unix = start_time.timestamp()
            total_time = (end_time - start_time).total_seconds() / 60
            elapsed_time = (cur_time - start_time).total_seconds() / 60
            percentage = elapsed_time / total_time * 100

            notified = False
            if init and diff_minutes <= 0:
                notified = True

            # Correct type for completed buildings
            completed_type = row['completedActivity.Type']

            if 'build' in row['completedActivity.SpriteName']:
                completed_type = 'UPGRADE'
            elif 'scan' in row['completedActivity.SpriteName']:
                completed_type = 'SCAN'
            elif 'illuvials' in row['completedActivity.SpriteName']:
                completed_type = 'RESEARCH'
            # if no resources have been generated then it is an extract
            elif row['fractionalGeneratedResources'] == '0':
                completed_type = 'EXTRACT'

            timers[f'{row["uid"]}-current'] = {'uid': row['uid'],
                                               'name': row['buildingTypeString'],
                                               'type': completed_type,
                                               'minutes': diff_minutes,
                                               'notified': notified,
                                               'percentage': percentage,
                                               'end': end_unix,
                                               'start': start_unix}

    # sort by minutes
    timers = {k: v for k, v in sorted(
        timers.items(), key=lambda item: item[1]['minutes'])}
    return timers



import requests
import os
import pyilz.refresh_token as refresh_token
import pyilz.get_token as get_token


def get_my_plots(api_key=None):
    """
    Returns a list of all plots owned by the user. Auth required.

    Args:
        api_key (str, optional): API key for authentication. Defaults to None.

    Returns:
        list: A list of all plots owned by the user.
    """
    API_KEY = get_token.get_token() if api_key is None else api_key
    url = f'http://api.illuvium-game.io/gamedata/api/zero/plots'

    response = requests.get(url, headers={'Authorization':
                            'Bearer ' + API_KEY})
    if response.status_code == 401:
        print('Token expired, refreshing...')
        API_KEY = refresh_token.refresh_token()
        response = requests.get(url, headers={'Authorization':
                                              'Bearer ' + API_KEY})
        os.environ['API_KEY'] = API_KEY

    return response.json()


def get_plot_metadata(land_id):
    """
    Returns a dictionary of metadata for the specified plot.

    Args:
        land_id (int): The ID of the plot to retrieve metadata for.

    Returns:
        dict: A dictionary containing metadata for the specified plot.
    """
    url = f'http://api.illuvium-game.io/gamedata/api/zero/plots/{land_id}/metadata'
    response = requests.get(url)
    return response.json()


if __name__ == '__main__':
    print(get_plot_metadata(53721))



import json
from pyilz.parse_land import parse_land, get_buildings_and_activities
from pyilz.get_timers import get_timers
from pyilz.get_resources import get_storage
from pyilz.get_plots import get_plot_metadata


def test_e2e():
    # Get list of plots
    with open('./tests/example_game_state.json', 'r') as f:
        plots = json.load(f)['data']

    land = plots[2]['data']
    landId = plots[2]['landId']
    plot_metadata = get_plot_metadata(landId)
    tier = plot_metadata['tier']
    region = plot_metadata['region']
    building_data = parse_land(land)

    paths, buildings, ca, aa, completed = get_buildings_and_activities(
        building_data)

    # Get timers from actions
    timers = get_timers(ca, aa, completed)

    storage = get_storage(building_data, tier)

    # If no exception, test passed
    assert True


if __name__ == '__main__':
    test_e2e()



import json
import math
import importlib.resources
import json
with importlib.resources.open_text("pyilz", "buildings.json") as file:
    buildings = json.load(file)['buildings']


def get_building_dimensions(name):
    """
    Returns the width and height of a building with the given name.

    Args:
        name (str): The name of the building.

    Returns:
        tuple: A tuple containing the width and height of the building.
               If the building is not found, returns a tuple with default
               values of 2 for width and height.
    """
    name = name.lower().replace(' ', '').replace('_', '').replace('-', '')
    for building in buildings:  # This should probably be a dict
        if building['nameId'] == name:
            return building['width'], building['height']
    return 2, 2


def get_building_starting_efficiency(name):
    """
    Returns the starting efficiency of a building with the given name.

    Args:
        name (str): The name of the building.

    Returns:
        int: The starting efficiency of the building, or 100 if not found.
    """
    name = name.lower().replace(' ', '').replace('_', '').replace('-', '')
    for building in buildings:  # This should probably be a dict
        if building['nameId'] == name:
            return building['efficiency'] if 'efficiency' in building else 100
    return 100


def get_building_radius(name):
    """
    Returns the radius of a building based on its name.

    Args:
        name (str): The name of the building.

    Returns:
        int: The radius of the building.
    """
    radius = 6
    if name == 'CONDENSER_PLANT' or name == 'PHOTODISINTEGRATION_PLANT' or name == 'SEQUESTRIAN_PLANT' or name == 'lcryptoncollider' or name == 'antisoloninverter' or name == 'hyperionlathe':
        radius = 7
    if name == 'MATERIALS_LAB':
        radius = 8
    if name == 'MARKETPLACE':
        radius = 12
    return radius


def get_building_storage(name: str, level: int, tier: int) -> tuple:
    """
    Returns the storage capacity and resource type of a building at a given level and tier.

    Args:
        name (str): The name of the building.
        level (int): The level of the building.
        tier (int): The tier of the land the building is on.

    Returns:
        tuple: A tuple containing the storage capacity (int) and resource type (str) of the building.
    """
    name = name.lower().replace(' ', '').replace('_', '').replace('-', '')
    land_tier_bonus_multiplier_percent = [0, 0, 33.33333333333, 100, 300, 900]
    for building in buildings:  # This should probably be a dict
        if building['nameId'] == name:
            details = building['details']
            for detail in details:
                if detail['level'] == level:
                    if 'storage' not in detail:
                        return 0, None
                    storage = detail['storage']
                    if storage is None:
                        return 0, None
                    amount = int(storage['amount'])
                    resource = storage['resource']
                    return math.ceil(amount * (1 + (land_tier_bonus_multiplier_percent[tier] / 100))), resource


def get_active_output(name: str, level: int, tier: int, efficiency: int = 100) -> int:
    """
    Returns the active output of a building with the given name, level, and tier.

    Args:
        name (str): The name of the building.
        level (int): The level of the building.
        tier (int): The tier of the building.
        efficiency (int, optional): The efficiency of the building, defaults to 100.

    Returns:
        int: The active output of the building.
    """
    name = name.lower().replace(' ', '').replace('_', '').replace('-', '')
    land_tier_bonus_multiplier_percent = [0, 0, 33.33333333333, 100, 300, 900]
    for building in buildings:  # This should probably be a dict
        if building['nameId'] == name:
            details = building['details']
            for detail in details:
                if detail['level'] == level:
                    activities = detail['activities']
                    active = activities['active']
                    if active is None:
                        return 0
                    amount = int(active['amount'])
                    return math.ceil((amount * efficiency / 100) * (1 + (land_tier_bonus_multiplier_percent[tier] / 100)))


def get_passive_output(name: str, level: int, tier: int, efficiency: int = 100) -> int:
    """
    Calculates the passive output of a building given its name, level, tier, and efficiency.

    Args:
        name (str): The name of the building.
        level (int): The level of the building.
        tier (int): The tier of the building.
        efficiency (int, optional): The efficiency of the building. Defaults to 100.

    Returns:
        int: The calculated passive output of the building. 
    """
    name = name.lower().replace(' ', '').replace('_', '').replace('-', '')
    land_tier_bonus_multiplier_percent = [0, 0, 33.33333333333, 100, 300, 900]
    for building in buildings:  # This should probably be a dict
        if building['nameId'] == name:
            details = building['details']
            for detail in details:
                if detail['level'] == level:
                    activities = detail['activities']
                    passive = activities['passive']
                    if passive is None:
                        return 0
                    amount = int(passive['amount'])
                    return math.ceil((amount * efficiency / 100) * (1 + (land_tier_bonus_multiplier_percent[tier] / 100)))


if __name__ == '__main__':
    print('-- Active --')
    print(
        f'Level 5 Hydrogen Pump on Tier 2 Land at 150% efficiency: {get_active_output("Hydrogen Pump", 5, 2, 150)}')
    print(
        f'Level 6 Sequestrian Plant on Tier 2 Land at 150% efficiency: {get_active_output("Sequestrian Plant", 6, 2, 150)}')
    print(
        f'Level 6 Sequestrian Plant on Tier 3 Land at 150% efficiency: {get_active_output("Sequestrian Plant", 6, 3, 150)}')
    print('-- Passive --')
    print(
        f'Level 5 Hydrogen Pump on Tier 2 Land at 150% efficiency: {get_passive_output("Hydrogen Pump", 5, 2, 150)}')
    print(
        f'Level 6 Sequestrian Plant on Tier 2 Land at 150% efficiency: {get_passive_output("Sequestrian Plant", 6, 2, 150)}')
    print(
        f'Level 6 Sequestrian Plant on Tier 3 Land at 150% efficiency: {get_passive_output("Sequestrian Plant", 6, 3, 150)}')



from pyilz.get_buildings import get_building_storage


def _get_storage_buildings(df):
    """
    Returns a filtered DataFrame containing only the storage buildings from the input DataFrame.

    Args:
        df (pandas.DataFrame): The input DataFrame containing the building data.

    Returns:
        pandas.DataFrame: A filtered DataFrame containing only the storage buildings.
    """
    storage_buildings_suffix = ['SILO', 'UNIT']
    # pop off _{number} from buildingTypeString and put into column Level
    df['Level'] = df['buildingTypeString'].str.extract(
        r'_(\d+)', expand=False).fillna(0).astype(int)
    # remove _{number} from buildingTypeString and put into column Type
    df['Type'] = df['buildingTypeString'].str.extract(
        r'(.+?)_\d+$', expand=False).str.rstrip('_')
    # get rows where Type ends with SILO or UNIT
    df = df[df['Type'].str.endswith(tuple(storage_buildings_suffix))]
    return df


def get_storage(parsed_land, tier=1):
    """
    Calculates the total amount of resources that can be stored in the player's storage buildings.

    Args:
        parsed_land (pandas.DataFrame): A DataFrame containing information about the player's land.
        tier (int, optional): The tier of the storage buildings to consider. Defaults to 1.

    Returns:
        dict: A dictionary containing the total amount of each resource that can be stored.
    """
    storage_buildings = _get_storage_buildings(parsed_land)
    storage = {
        'hydrogen': 0,
        'silicon': 0,
        'carbon': 0,
        'crypton': 0,
        'hyperion': 0,
        'solon': 0,
    }
    for _, row in storage_buildings.iterrows():
        amount, resource = get_building_storage(
            row['Type'], row['Level'], tier)
        storage[resource] += amount
    return storage



import requests
import os


def refresh_token(ref_token=None):
    """
    Returns a new ID token using the provided refresh token or the refresh token from the environment.

    Args:
        ref_token (str, optional): The refresh token to use. Defaults to None.

    Raises:
        Exception: If no refresh token is found.

    Returns:
        str: The new ID token.
    """
    REFRESH_TOKEN = os.getenv(
        'REFRESH_TOKEN') if ref_token is None else ref_token
    if REFRESH_TOKEN is None:
        raise Exception(
            'No refresh token found. Please provide a refresh token.')

    response = requests.post('https://cognito-idp.us-east-1.amazonaws.com/', headers={
        'authority': 'cognito-idp.us-east-1.amazonaws.com',
        'content-type': 'application/x-amz-json-1.1',
        'x-amz-target': 'AWSCognitoIdentityProviderService.InitiateAuth'
    },
        json={
        'ClientId': 'kutk35sfb1haiut4s7vmda4e8',
        'AuthFlow': 'REFRESH_TOKEN_AUTH',
        'AuthParameters': {
            'REFRESH_TOKEN': REFRESH_TOKEN,
        }
    }, timeout=5)

    if response.status_code != 200:
        raise Exception(
            f'Failed to refresh token. {response.status_code} {response.text}')

    authentication_result = response.json()['AuthenticationResult']
    # access_token = authentication_result['AccessToken']
    # expires_in = authentication_result['ExpiresIn']
    id_token = authentication_result['IdToken']
    # token_type = authentication_result['TokenType']

    return id_token
